///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ILNumerics.Misc {
    /// <summary>
    /// Manages local array caches, singleton instance is accessed via: ILNumerics.ILArrayCache&lt;T;gt;.Manager
    /// </summary>
    public class ILCacheManager {

        private static ILCacheManager s_manager;
        /// <summary>
        /// Singleton instance of a cache manager, used to retrieve local array caches 
        /// </summary>
        public static ILCacheManager Manager {
            get {
                if (s_manager == null) {
                    s_manager = new ILCacheManager();
                }
                return s_manager;
            }
        }

        IList<IDisposable> m_caches = new List<IDisposable>(); 

        internal ILCacheManager() {

        }

        /// <summary>
        /// Create a new ILArrayCache and add to managed caches
        /// </summary>
        /// <returns>The new cache</returns>
        public ILArrayCache GetCache() {
            ILArrayCache ret = new ILArrayCache(); 
            m_caches.Add(ret); 
            return ret; 
        }

        /// <summary>
        /// Dispose all managed caches
        /// </summary>
        public void FreeAllCaches() {
            foreach (IDisposable d in m_caches) {
                if (d != null) {
                    d.Dispose(); 
                }
            }
        }



    }
}
