///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ILNumerics {
    internal class ILScopeList<T> {

        #region attributes
        T[] m_data; 
        int m_count; 
        #endregion

        #region properties 
        public int Count {
            get { return m_count; }
            set { m_count = value; }
        }
        #endregion

        #region
        public ILScopeList() {
            m_data = new T[10]; 
            m_count = 0; 
        }
        public ILScopeList(int capacity) {
            m_data = new T[capacity]; 
            m_count = 0; 
        }
        #endregion

        #region public interface
        public T Pop() {
            if (m_count == 0) throw new IndexOutOfRangeException("The ILScopeList is empty. Pop cannot be completed.");
            T ret = m_data[--m_count]; 
            m_data[m_count + 1] = default(T); 
            return ret; 
        }
        public void Push(T element) {
            ensureLength(m_count); 
            m_data[m_count++] = element; 
        }
        public T Peek() {
            if (m_count == 0) throw new IndexOutOfRangeException("The ILScopeList is empty. Peek cannot be completed.");
            return m_data[m_count-1]; 
        }
        /// <summary>
        /// peek element at distance 'index' from END!!
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public T Peek(int index) {
            index = m_count - 1 - index; 
            return m_data[index]; 
        }
        #endregion

        #region private helper 
        private void ensureLength(int newIndex) {
            while (newIndex >= m_data.Length) {
                T[] newArr = new T[m_data.Length * 2]; 
                System.Array.Copy(m_data,newArr,m_count); 
                m_data = newArr; 
            }
        }
        #endregion

    }
}
